#
# MD5 auth
#

# $Id$

require "digest/md5"
require "digest/sha1"
require "digest/sha2"
require "digest/rmd160"
require "util.rb"

class DnsAuthGeneric
  def initialize(method, type, answer)
    @type = type

    name = dnsstr_to_str(answer["a"]).downcase
    Regexp.new("^sec\-auth\-#{type}\-\-\.([0-9a-f\.]+)\.--\.([a-z0-9\-\.]+)$") =~ name #"
    sum = $1
    base = $2
    ttl = "\0\0\0\0"

    if sum == nil || base == nil
      @format_valid = false
      return
    end

    @format_valid = true

    packet = str_to_dnsstr(base) + answer["a_type"] + answer["a_class"] +
      ttl + [answer["data"].size].pack("n") + answer["data"]

    @digest = method.new(packet)
    @sig    = sum.tr(".", "")

  end

  def valid()
    if @format_valid == false
      ML.log(@type.upcase + " bad format")
      return false
    end

    if @digest.hexdigest == @sig
      ML.log(@type.upcase + " good")
      return true
    else
      ML.log(@type.upcase + " bad")
      return false
    end
  end
end

class DnsAuthMD5 < DnsAuthGeneric
  def initialize(answer)
    super(Digest::MD5, "md5", answer)
  end
end

class DnsAuthSHA1 < DnsAuthGeneric
  def initialize(answer)
    super(Digest::SHA1, "sha1", answer)
  end
end

class DnsAuthSHA256 < DnsAuthGeneric
  def initialize(answer)
    super(Digest::SHA256, "sha256", answer)
  end
end

class DnsAuthSHA384 < DnsAuthGeneric
  def initialize(answer)
    super(Digest::SHA384, "sha384", answer)
  end
end

class DnsAuthSHA512 < DnsAuthGeneric
  def initialize(answer)
    super(Digest::SHA512, "sha512", answer)
  end
end

class DnsAuthRMD160 < DnsAuthGeneric
  def initialize(answer)
    super(Digest::RMD160, "rmd160", answer)
  end
end

# end
